/*
  Calisto - An internet Talker
  Copyright (C) 1998-2001 Peter Howkins, Matthew Howkins and Simon Howkins

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,     
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License   
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Email: calisto@calisto.cx

  $Id: dllist.h,v 1.3 2000/12/29 20:54:40 peter Exp $

 */

#ifndef dllist_h
#define dllist_h

#ifndef OFFSETOF
#define OFFSETOF(pstructcast, linkfield) \
  ((char *) &(((pstructcast) 0)->linkfield))
#endif

typedef struct listnode listnode;

struct listnode {
  listnode *prev;
  listnode *next;
};

typedef struct listhead listhead;

struct listhead {
  listnode head;
  listnode tail;
};

#define LIST_INIT(listhead) (                          \
  (listhead).head.next = &(listhead).tail,             \
  (listhead).tail.prev = &(listhead).head,             \
  (listhead).head.prev = (listhead).tail.next = NULL )

/* listnode is a pointer to a listnode */
#define LIST_UNLINK_NODE(listnode) (          \
  (listnode)->next->prev = (listnode)->prev,  \
  (listnode)->prev->next = (listnode)->next )

#define LIST_IS_EMPTY(listhead) ((listhead).head.next == &(listhead).tail)

#define LIST_LINK_NODE_AT_END(listhead,listnode) \
  ((listnode)->next = &(listhead).tail,          \
  (listnode)->prev = (listhead).tail.prev,       \
  (listhead).tail.prev->next = (listnode),       \
  (listhead).tail.prev = (listnode))

#define LIST_LINK_NODE_BEFORE(newnode, curnode) \
  ((newnode)->next = curnode,                   \
  (newnode)->prev = curnode->prev,              \
  (curnode)->prev->next = (newnode),            \
  (curnode)->prev = (newnode))

#define LIST_NODE_IS_REAL(listnode) ((listnode)->prev && (listnode)->next)

#define LIST_GET_DATA(listnode,pstructcast,linkfield) \
  ((pstructcast) ((char *) listnode - OFFSETOF(pstructcast, linkfield)))

unsigned list_len(const listhead *);

void list_qsort_n(listnode *pivot,
                  int (cmp)(const listnode *, const listnode *),
                  unsigned len);

void list_qsort(listhead *head,
                int (cmp)(const listnode *, const listnode *));

#endif /* dllist_h */
