/*
  Calisto - An internet Talker
  Copyright (C) 1998-2001 Peter Howkins, Matthew Howkins and Simon Howkins

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,     
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License   
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Email: calisto@calisto.cx

  $Id: globals.h,v 1.12 2000/12/29 20:55:21 peter Exp $

 */

#ifndef globals_h
#define globals_h

#include <time.h>

#include "dllist.h"
#include "pool.h"
#include "structs.h"
#include "log.h"

/* **** Configurable parameters - don't change unless you know what you are
        doing */
/* the salt for the crypt() function used for password comparison and 
   generation */
#define CRYPT_SALT "Ca"

/* location of the player file directory - requires trailing / */
#define PLAYER_DIR "lib/players/"


/* **** No configuration below this point */
/* this is used regardless of whether plugins are used or not */
#define NO_PLUGIN_EXTRA_DATA -1

/* program version */
extern const unsigned version;

/* the time at which the talker was booted */
extern time_t starttime;

/* the size in bytes of all the data that the plugins have requested */
extern size_t plugin_extra;

/* Playing character list, and descriptor/connection list */
extern listhead AllConns;
extern listhead AllPlayers;

/* if TRUE, talker will exit main loop and shutdown */
extern bool sdown;

/* the memory pool of descriptors, used for memory stats and allocating new
   descriptors */
extern pool *descriptor_pool;

/* memory pool of the plugins */
#ifdef USE_PLUGINS
extern pool *plugin_pool;
#endif /* USE_PLUGINS */

/* different modes in which the talker can run */
#define TALKER_OPEN      1
#define TALKER_RESIDENCY 2
#define TALKER_INVITE    3 
#define TALKER_SU_INVITE 4 

/* Values from .ini file */
extern char talker_name[];
extern unsigned port;
extern char admin_name[];
extern bool use_net_lookups;
extern unsigned idle_boot_time;
extern unsigned max_connections;
extern unsigned talker_level;

#endif /* globals_h */
