/*
  Calisto - An internet Talker
  Copyright (C) 1998-2001 Peter Howkins, Matthew Howkins and Simon Howkins

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,     
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License   
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Email: calisto@calisto.cx

  $Id: log.c,v 1.7 2002/12/02 19:44:32 peter Exp $

 */
static const char rcsid[] = "$Id: log.c,v 1.7 2002/12/02 19:44:32 peter Exp $";

#include <stdarg.h>
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#include <time.h>

#include "config.h"
#include "debug.h"
#include "log.h"
#include "mem.h"
#include "msnprintf.h"
#include "strplus.h"

LOGF *usage, *debug, *bug, *idea, *typo;

void log(LOGF *log, const char *sFormat, ...)
{
  va_list ptr;
  time_t now;
  struct tm now2;
  char caBuffer[256];
  FILE *f;

  ASSERT(log);
  ASSERT(sFormat);

  /* Prepare formatted time string */
  time(&now);
  now2 = *localtime(&now);
  strftime(caBuffer, 256, log->dateformat, &now2);
  /* Open file and output log info */
  f = fopen(log->filename, "a");
  if (!f) {
    perror("Could not open logfile");
  } else {
    fprintf(f, "%s ", caBuffer);
    va_start(ptr, sFormat);
    vfprintf(f, sFormat, ptr);
    va_end(ptr);
    fprintf(f, "\n");
    fclose(f);
  }
  /* Output to stdout also */
/*  printf("%s ", caBuffer);
  va_start(ptr, sFormat);
  vprintf(sFormat, ptr);
  va_end(ptr);
  printf("\n");*/
}

LOGF *log_init(const char *sFilename, const char *sDateformat)
{
  LOGF *log;

  ASSERT(sFilename);
  ASSERT(sDateformat);

  log = MEM_MALLOC(sizeof(LOGF));
  if (log) {
    STRNCOPY(log->filename, sFilename, sizeof(log->filename));
    msnprintf(log->dateformat, sizeof(log->dateformat), "[%s]", sDateformat);
  } else {
    fprintf(stderr, "Unable to create log '%s'\n", sFilename);
  }
  return log;
}

void log_close(LOGF *log)
{
  if(log) {
    MEM_FREE(log);
  }
}
