/*
  Calisto - An internet Talker
  Copyright (C) 1998-2001 Peter Howkins, Matthew Howkins and Simon Howkins

  This program is free software; you can redistribute it and/or modify
  it under the terms of the GNU General Public License as published by
  the Free Software Foundation; either version 2 of the License, or
  (at your option) any later version.

  This program is distributed in the hope that it will be useful,     
  but WITHOUT ANY WARRANTY; without even the implied warranty of
  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE.  See the
  GNU General Public License for more details.
  
  You should have received a copy of the GNU General Public License   
  along with this program; if not, write to the Free Software
  Foundation, Inc., 675 Mass Ave, Cambridge, MA 02139, USA.

  Email: calisto@calisto.cx

  $Id: socks.c,v 1.6 2002/12/02 19:41:04 peter Exp $

 */
static const char rcsid[] = "$Id: socks.c,v 1.6 2002/12/02 19:41:04 peter Exp $";

/* Ansi Includes */
#include <errno.h>
#include <stdio.h>
#include <stdarg.h>
#include <stdlib.h>
#include <string.h>
/* Unix includes */
#include "sys/types.h"
#include "sys/socket.h"
#include "netinet/in.h"
#include "unistd.h"
/* Our includes */
#include "log.h"
#include "globals.h"
#include "msnprintf.h"
#include "structs.h"

/* Gets as many characters as possible into the buffer.
   There may be nothing, some characters but not a whole line,
   possibly a whole line, or several lines including bits.

   Returns -1 for problem with socket
           0  for no characters available, but connection OK
           >0 number of characters placed in buffer - no guarantee of how many lines
*/
int get_some_from_socket(int sfd, char *buffer, size_t bufsize)
{
  int size, count = 0;
  char c;

  while (count < bufsize) {
    size = recv(sfd, &c, 1, 0);
    switch (size) {
      case -1:
        /* No character read, but connection OK */
        if(EAGAIN == errno) {
          return count;
        } else {
          return -1;
        }
      case 0:  /* No character read because connection broken */
        return -1;
    }
    /* Character read OK */
    *buffer++ = c;
    count++;
  }
  return count;
}

int send_to_socket(int sfd, const char *format, ...) {
  va_list ptr;
  char gap[OUTPUT_BUFFER];
  int check;

  va_start(ptr, format);
  mvsnprintf(gap, sizeof(gap), format, ptr);
  va_end(ptr);

  check = send(sfd, gap, strlen(gap),0);
  if (check == -1) {
    log(usage, "Error writing to socket %d: %s", sfd, strerror(errno));
    return 1;
  }
  return 0;
}

char *ip4_to_string(char *buffer, size_t bufsize,
                    const struct in_addr *ip_addr)
{
  unsigned long address = ntohl(ip_addr->s_addr);

  msnprintf(buffer, bufsize, "%lu.%lu.%lu.%lu", (address >> 24) & 0xff,
            (address >> 16) & 0xff, (address >> 8) & 0xff, address & 0xff);

  return buffer;
}
